"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Server = void 0;
const apollo_server_express_1 = require("apollo-server-express");
const body_parser_1 = require("body-parser");
const express_1 = __importDefault(require("express"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const graphql_depth_limit_1 = __importDefault(require("graphql-depth-limit"));
const merge_1 = require("@graphql-tools/merge");
const util_1 = require("./util");
const apollo_server_plugins_1 = require("./apollo_server_plugins");
const errors_1 = require("./errors");
const express_middleware_1 = require("./express_middleware");
class Server {
    constructor(schemas, config) {
        this.app = express_1.default();
        this.config = config;
        this.schemas = schemas;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            let allowList;
            const plugins = [];
            const validationRules = [];
            if (this.config.allowListPath) {
                if (this.config.allowIntrospection === true) {
                    throw new errors_1.IntrospectionNotPermitted('allowListPath');
                }
                try {
                    const file = yield fs_extra_1.default.readFile(this.config.allowListPath, 'utf8');
                    allowList = JSON.parse(file);
                    this.app.use('/', body_parser_1.json(), express_middleware_1.allowListMiddleware(allowList));
                    console.log('The server will only allow only operations from the provided list');
                }
                catch (error) {
                    console.error(`Cannot read or parse allow-list JSON file at ${this.config.allowListPath}`);
                    throw error;
                }
            }
            if (this.config.prometheusMetrics) {
                if (this.config.tracing === false) {
                    throw new errors_1.TracingRequired('Prometheus');
                }
                plugins.push(apollo_server_plugins_1.prometheusMetricsPlugin(this.app));
                console.log('Prometheus metrics will be served at /metrics');
            }
            if (this.config.queryDepthLimit) {
                validationRules.push(graphql_depth_limit_1.default(this.config.queryDepthLimit));
            }
            this.apolloServer = new apollo_server_express_1.ApolloServer({
                cacheControl: this.config.cacheEnabled ? { defaultMaxAge: 20 } : undefined,
                introspection: this.config.allowIntrospection,
                playground: this.config.allowIntrospection,
                plugins,
                validationRules,
                schema: merge_1.mergeSchemas({
                    schemas: this.schemas
                }),
                tracing: this.config.tracing
            });
            this.apolloServer.applyMiddleware({
                app: this.app,
                cors: this.config.allowedOrigins ? {
                    origin: this.config.allowedOrigins
                } : undefined,
                path: '/'
            });
        });
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            this.httpServer = yield util_1.listenPromise(this.app, this.config.apiPort, this.config.listenAddress);
            console.log(`GraphQL HTTP server at http://${this.config.listenAddress}:` +
                `${this.config.apiPort}${this.apolloServer.graphqlPath} started`);
        });
    }
    shutdown() {
        if (this.httpServer !== undefined) {
            this.httpServer.close();
            console.log(`GraphQL HTTP server at http://${this.config.listenAddress}:` +
                `${this.config.apiPort}${this.apolloServer.graphqlPath} shutting down`);
        }
    }
}
exports.Server = Server;
//# sourceMappingURL=Server.js.map